<?php

namespace Drupal\a12s_maps_sync\Commands;

use Drupal\a12s_maps_sync\AutoConfigManager;
use Drupal\a12s_maps_sync\BatchService;
use Drupal\a12s_maps_sync\Entity\Converter;
use Drupal\a12s_maps_sync\Entity\Profile;
use Drupal\a12s_maps_sync\Exception\MapsException;
use Drupal\Core\Entity\EntityStorageException;
use Drush\Commands\DrushCommands;

/**
 * A Drush commandfile for a12s maps sync.
 */
class A12sMapsSyncCommands extends DrushCommands {

  /**
   * @param AutoConfigManager $autoConfigManager
   */
  public function __construct(
    protected AutoConfigManager $autoConfigManager
  ){
    parent::__construct();
  }

  /**
   * Rollback the given profile.
   *
   * @param string $profile
   *   The profile name.
   *
   * @command a12s_maps_sync:rollback:profile
   * @aliases amsrp
   *
   * @usage a12s_maps_sync:rollback:profile my_profile
   */
  public function rollbackProfile(string $profile): void {
    $messenger = \Drupal::messenger();

    // Load the profile.
    try {
      $profile = Profile::load($profile);

      $messenger->addMessage('Rollback profile ' . $profile->label());

      /** @var \Drupal\a12s_maps_sync\Entity\Converter $converter */
      foreach ($profile->getConverters() as $converter) {
        $messenger->addMessage("-- Rollback converter {$converter->label()}");

        $results = $converter->rollback();

        $messenger->addStatus('-- ' . count($results) . ' elements deleted');
      }
    }
    catch (EntityStorageException $e) {
      $messenger->addError("The profile $profile does not exist");
      return;
    }

    $messenger->addStatus('Rollback finished');
  }

  /**
   * Rollback the given converter.
   *
   * @param string $converter
   *   The converter name.
   *
   * @command a12s_maps_sync:rollback:converter
   * @aliases amsrc
   *
   * @usage a12s_maps_sync:rollback:converter my_converter
   */
  public function rollbackConverter(string $converter): void {
    $results = [];

    $messenger = \Drupal::messenger();

    // Try to load the converter.
    try {
      $converter = Converter::load($converter);
      $messenger->addMessage('Rollback converter ' . $converter->label());

      $results = $converter->rollback();
    } catch (EntityStorageException $e) {
      $messenger->addError($e->getMessage());
    }

    if (!empty($results)) {
      $messenger->addStatus('Rollback finished');
      $messenger->addStatus(count($results) . ' elements deleted');
    }
  }

  /**
   * Import the given profile.
   *
   * @param string $profile
   *   The profile name.
   * @param array $options
   *   The options array.
   *
   * @option int limit
   *   The limit of elements to process.
   * @option bool force
   *   Force the import despite the lock status.
   * @option bool all
   *   Force the import of all data
   *
   * @command a12s_maps_sync:import:profile
   * @aliases amsip
   *
   * @usage a12s_maps_sync:import:profile my_profile --limit=20 --force=true
   */
  public function importProfile(string $profile, array $options = ['limit' => NULL, 'force' => FALSE, 'all' => FALSE]): void {
    // Load the profile.
    try {
      $profile = Profile::load($profile);

      $lockName = 'a12s_maps_sync:lock:profile:' . $profile->getPythonProfileId();

      $config = \Drupal::config('a12s_maps_sync.settings');

      if ((bool) $config->get('ignore_lock') === TRUE || $options['force'] || !\Drupal::state()->get($lockName)) {
        // Set the lock.
        \Drupal::state()->set($lockName, time());

        $batch = BatchService::getProfileImportBatchDefinition(
          $profile,
          $options['limit'] ?? $config->get('import_batch_size'),
          $options['all'],
        );

        batch_set($batch);
        drush_backend_batch_process();
      }
      else {
        $this->logger()->error("The {$profile->getPythonProfileId()} profile is locked. You can remove the lock with the following command: drush amsrl {$profile->getPythonProfileId()}");
      }
    }
    catch (EntityStorageException $e) {
      $this->logger()->error("The profile $profile does not exist");
      exit(500);
    }
  }

  /**
   * Import the given converter.
   *
   * @param string $converter
   *   The converter name.
   * @param array $options
   *   The options array.
   *
   * @option int limit
   *   The limit of elements to process.
   * @option bool force
   *   Force the import despite the lock status.
   * @option bool all
   *   Force the import of all data
   *
   * @command a12s_maps_sync:import:converter
   * @aliases amsic
   *
   * @usage a12s_maps_sync:import:converter my_converter --limit=20 --force
   */
  public function importConverter(string $converter, array $options = ['limit' => NULL, 'force' => 'false', 'all' => FALSE]): void {
    $converter = Converter::load($converter);

    $profile = $converter->getProfile();
    $lockName = 'a12s_maps_sync:lock:profile:' . $profile->getPythonProfileId();

    $config = \Drupal::config('a12s_maps_sync.settings');

    if ((bool) $config->get('ignore_lock') === TRUE || $options['force'] || !\Drupal::state()->get($lockName)) {
      // Set the lock.
      \Drupal::state()->set($lockName, time());

      $limit = $options['limit'] ?? $config->get('import_batch_size');

      // Try to load the converter.
      try {
        $batch = BatchService::getConverterImportBatchDefinition($converter, $limit, $options['all']);

        batch_set($batch);
        drush_backend_batch_process();
      } catch (\Exception $e) {
        $this->logger()->error($e->getMessage());
        exit(500);
      }
    }
    else {
      $this->logger()->error("The {$profile->getPythonProfileId()} profile is locked. You can remove the lock with the following command: drush amsrl {$profile->getPythonProfileId()}");
    }
  }

  /**
   * @param string $profile
   *   The profile machine name.
   *
   * @command a12s_maps_sync:release_lock
   * @aliases amsrl
   *
   * @usage a12s_maps_sync:release_lock my_converter
   */
  public function releaseLock(string $profile): void {
    $this->output()->writeln('Releasing lock for profile ' . $profile);

    \Drupal::state()->delete('a12s_maps_sync:lock:profile:' . $profile);

    $this->logger()->success('Lock released');
  }

  /**
   * @param string $converter
   *   The converter machine name.
   *
   * @command a12s_maps_sync:auto_config_converter
   * @aliases amsacc
   *
   * @usage a12s_maps_sync:auto_config_converter my_converter
   */
  public function autoConfigConverter(string $converter): void {
    $this->output()->writeln('Auto configuration for converter ' . $converter);

    $converter = Converter::load($converter);

    $autoConfig = $converter->getAutoConfig();
    $attributeSets = $autoConfig['attribute_sets'] ?? [];

    if (empty($attributeSets)) {
      $this->output()->writeln('No auto configuration for this converter');
    }
    else {
      $this->autoConfigManager->processConverter($converter);
    }
  }

  /**
   * @param string $profile
   *   The profile machine name.
   *
   * @command a12s_maps_sync:auto_config_profile
   * @aliases amsacp
   *
   * @usage a12s_maps_sync:auto_config_profile my_profile
   */
  public function autoConfigProfile(string $profile): void {
    // @todo use batch.
    $this->output()->writeln('Auto configuration for profile ' . $profile);

    $profile = Profile::load($profile);

    foreach ($profile->getConverters() as $converter) {
      $autoConfig = $converter->getAutoConfig();
      if (empty($autoConfig['attribute_sets'])) {
        continue;
      }

      $this->output()->writeln('-- Auto configuration for converter ' . $converter->label());
      $this->autoConfigManager->manageAttributeSets($converter);
    }
  }

  /**
   * @param string $entityType
   * @param int $entityId
   * @param array $options
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\a12s_maps_sync\Exception\MapsException
   * @command a12s_maps_sync:import_entity
   * @aliases amsie
   *
   * @option bool with-dependencies
   *
   * @usage a12s_maps_sync:import_entity node 123
   */
  public function importEntity(string $entityType, int $entityId, array $options = ['profile' => NULL, 'converter' => NULL, 'with-dependencies' => FALSE]): void {
    // Load the entity.
    $entity = \Drupal::entityTypeManager()->getStorage($entityType)->load($entityId);

    if (!$entity) {
      throw new MapsException("No entity found ($entityType $entityId)");
    }

    try {
      $profile = !is_null($options['profile']) ? Profile::load($options['profile']) : NULL;
      $converter = !is_null($options['converter']) ? Converter::load($options['converter']) : NULL;

      $batch = BatchService::getEntityImportBatchDefinition($entity, $options['with-dependencies'], $profile, $converter);

      batch_set($batch);
      drush_backend_batch_process();
    } catch (\Exception $e) {
      $this->logger()->error($e->getMessage());
      exit(500);
    }
  }

  /**
   * @param string $mapsType
   * @param int $objectId
   * @param string $converterId
   * @param array $options
   *
   * @command a12s_maps_sync:import_object
   * @aliases amsio
   *
   * @option bool with-dependencies
   *
   * @usage a12s_maps_sync:import_object object 123 watch
   */
  public function importObject(string $mapsType, int $objectId, string $converterId, array $options = ['with-dependencies' => FALSE]): void {
    try {
      $converter = Converter::load($converterId);
      $batch = BatchService::getObjectImportBatchDefinition($mapsType, $objectId, $converter, $options['with-dependencies']);

      if ($batch) {
        batch_set($batch);
        drush_backend_batch_process();
      }
    } catch (\Exception $e) {
      $this->logger()->error($e->getMessage());
      exit(500);
    }
  }

  /**
   * @param string $profile
   *   The profile machine name.
   *
   * @command a12s_maps_sync:force_profile_reimport
   * @aliases amsfpr
   *
   * @usage a12s_maps_sync:force_profile_reimport my_profile
   */
  public function forceProfileReimport(string $profile): void {
    $profile = Profile::load($profile);
    if ($profile) {
      $this->output()->writeln('Force reimport for profile ' . $profile->label());

      foreach ($profile->getConverters() as $converter) {
        $this->output()->writeln(' - Force reimport for converter ' . $converter->label());
        \Drupal::state()->delete($converter->getLastImportedStateKey());
      }
    }
  }

  /**
   * @param string $converter
   *   The converter machine name.
   *
   * @command a12s_maps_sync:force_converter_reimport
   * @aliases amsfcr
   *
   * @usage a12s_maps_sync:force_converter_reimport my_converter
   */
  public function forceConverterReimport(string $converter): void {
    $converter = Converter::load($converter);
    if ($converter) {
      $this->output()->writeln('Force reimport for converter ' . $converter->label());
      \Drupal::state()->delete($converter->getLastImportedStateKey());
    }
  }

}
